{% extends "gourde/base.html" %}

{% block title %}Home{% endblock %}

{% block content %}
    <h1>Home</h1>
    <div class="progress">
        <div class="progress-bar progress-bar-striped progress-bar-animated" role="progressbar" aria-valuenow="75"
             aria-valuemin="0" aria-valuemax="100" style="width: 75%"></div>
    </div>
{% endblock %}